/*
 * IRC - Internet Relay Chat, ircd/m_vctrl.c
 * Copyright (C) 1990 Jarkko Oikarinen and
 *                    University of Oulu, Computing Center
 *
 * See file AUTHORS in IRC package for additional names of
 * the programmers.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: m_vctrl.c,v 1.1 2003/02/20 23:05:15 cometen Exp $
 */
#include "config.h"

#include "s_serv.h"
#include "IPcheck.h"
#include "channel.h"
#include "client.h"
#include "gline.h"
#include "hash.h"
#include "ircd.h"
#include "ircd_alloc.h"
#include "ircd_defs.h"
#include "ircd_features.h"
#include "ircd_reply.h"
#include "ircd_string.h"
#include "ircd_snprintf.h"
#include "ircd_xopen.h"
#include "jupe.h"
#include "list.h"
#include "match.h"
#include "msg.h"
#include "msgq.h"
#include "numeric.h"
#include "numnicks.h"
#include "parse.h"
#include "querycmds.h"
#include "s_bsd.h"
#include "s_conf.h"
#include "s_debug.h"
#include "s_misc.h"
#include "s_user.h"
#include "send.h"
#include "struct.h"
#include "sys.h"
#include "userload.h"

#include <assert.h>
#include <stdlib.h>
#include <string.h>


/*
 * ms_vctrl()
 *
 * Ported From Ultimate IRCd
 *
 * by HAMLET (From Ultimate Ircd)
 * Based on netinfo by Stskeeps but designed to maintain network
 * compactibility on several levels and to make sure all servers on the network
 * are running with the same network settings where it is required that they stay the same.
 * vctrl is also used to check before any client information is actually passed on so that
 * the servers wont have to sync several 100ds of users and channels before they find out oh crap
 * were not compactible with eachother, saves bandwith aswell.
 *
 *  parv[0] = sender prefix
 *  parv[1] = nickname length
 *  parv[2] = username length
 *  parv[3] = hostname length
 *  parv[4] = account length
 *  parv[5] = real name length
 *  parv[6] = passwd length
 *  parv[7] = sockip length
 *  parv[8] = topic length
 *  parv[9] = away length
 *  parv[10] = network
 *  parv[11] = Reserved for future extentions
 *  parv[12] = Reserved for future extentions
 *  parv[13] = Reserved for future extentions
 *  parv[14] = Reserved for future extentions
 *  parv[15] = Reserved for future extentions
 */

int ms_vctrl (struct Client* cptr, struct Client* sptr, int parc, char* parv[])
{

  if (GotVCtrl(cptr)) {
     sendto_opmask_butone(0, SNO_OLDSNO, "Already got VCTRL from Link %s", cli_name(cptr));
     return 0;
  }

  /*
   * Variable Processing
   */

  if (atoi(parv[1]) != NICKLEN)
    return exit_client(cptr, cptr, &me, "Nickname Length Mismatch");

  if (atoi(parv[2]) != USERLEN)
    return exit_client(cptr, cptr, &me, "Username Length Mismatch");

  if ((atoi(parv[3]) != HOSTLEN))
    return exit_client(cptr, cptr, &me, "Hostname Length Mismatch");

  if ((atoi(parv[4]) != ACCOUNTLEN))
    return exit_client(cptr, cptr, &me, "Account Length Mismatch");

  if ((atoi(parv[5]) != REALLEN))
    return exit_client(cptr, cptr, &me, "Real Name Length Mismatch");

  if ((atoi(parv[6]) != PASSWDLEN))
    return exit_client(cptr, cptr, &me, "Password Length Mismatch");

  if ((atoi(parv[7]) != SOCKIPLEN))
    return exit_client(cptr, cptr, &me, "Sock IP Length Mismatch");

  if ((atoi(parv[8]) != TOPICLEN))
    return exit_client(cptr, cptr, &me, "Topic Length Mismatch");

  if ((atoi(parv[9]) != AWAYLEN))
    return exit_client(cptr, cptr, &me, "Away Length Mismatch");

  if (0 != ircd_strcmp(feature_str(FEAT_NETWORK), parv[10]))
    return exit_client(cptr, cptr, &me, "Network Name Mismatch");


  /* Variables Fine, Continue Link */
  SetVCtrl(cptr);
  return 0;
}


